package com.hai.service.pay.impl;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.dao.HighUserCouponMapper;
import com.hai.entity.*;
import com.hai.service.HighCouponCodeService;
import com.hai.service.HighOrderService;
import com.hai.service.HighUserService;
import com.hai.service.pay.PayService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/27 00:35
 */
@Service("goodsOrderService")
public class GoodsOrderServiceImpl implements PayService {

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private HighCouponCodeService highCouponCodeService;

    @Resource
    private HighUserService highUserService;

    @Resource
    private HighUserCouponMapper highUserCouponMapper;

    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public void paySuccess(Map<String, String> map, String payType) throws Exception {
        if (payType.equals("Alipay")) {
            // 支付宝支付 todo 暂未开发
            return;
        }
        if (payType.equals("WechatPay")) {
            // 查询订单信息
            HighOrder order = highOrderService.getOrderByOrderNo(map.get("out_trade_no"));
            if (order == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_FOUND_ORDER, "");
            }
            order.setPaySerialNo(map.get("transaction_id")); // 支付流水号
            order.setPayRealPrice(new BigDecimal(map.get("total_fee")).divide(new BigDecimal("100"))); // 实付金额
            order.setPayTime(new Date()); // 支付时间
            order.setPayModel(2);         // 支付模式：1 金币，2 第三方平台，3 混合
            order.setPayType(2);          // 支付方式： 1：支付宝  2：微信  3：金币
            order.setOrderStatus(2);      // 订单状态：1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消

            for (HighChildOrder highChildOrder : order.getHighChildOrderList()) {
                highChildOrder.setChildOrdeStatus(2);  // 子订单状态：1 待支付  2 已支付  3.已完成  4. 已退款  5.已取消

                // 商品类型   商品类型   1:卡卷   2:金币充值
                if (highChildOrder.getGoodsType() == 1) {
                    HighCouponCode code = highCouponCodeService.getCouponCodeByOrderId(highChildOrder.getId());
                    code.setStatus(2);  // 状态：1.待销售 2.未使用 3.已使用  99.预支付
                    highCouponCodeService.updateCouponCode(code);

                    // 卡卷关联用户
                    HighUserCoupon highUserCoupon = new HighUserCoupon();
                    highUserCoupon.setMerchantId(code.getMerchantId());
                    highUserCoupon.setCouponId(code.getCouponId());
                    highUserCoupon.setUserId(order.getMemId());
                    highUserCoupon.setCouponCodeId(code.getId());
                    highUserCoupon.setUseEndTime(code.getUseEndTime());
                    highUserCoupon.setStatus(1);  // 状态  0:已过期  1：未使用  2：已使用
                    highUserCouponMapper.insert(highUserCoupon);
                }

                if (highChildOrder.getGoodsType() == 2) {
                    // 金币 1：100
                    Integer goldNum = new BigDecimal(highChildOrder.getTotalPrice().toString()).multiply(new BigDecimal("100")).intValue();
                    highUserService.goldHandle(highChildOrder.getGoodsId(), goldNum, 1, 1, highChildOrder.getId());
                }
            }

            highOrderService.updateOrder(order);

        }
    }
}
